<?php
/**
 * Manages interactions with the admin.
 *
 * @package Tasty_Recipes
 */

namespace Tasty_Recipes;

use Tasty\Framework\Utils\Url;
use Tasty_Recipes;

/**
 * Manages interactions with the admin.
 */
class Admin {

	/**
	 * Capability required to manage settings.
	 *
	 * @var string
	 */
	const CAPABILITY = 'manage_options';

	/**
	 * Plugin name for EDD registration.
	 *
	 * @var string
	 */
	const ITEM_AUTHOR = 'Tasty Recipes';

	/**
	 * Plugin author for EDD registration.
	 *
	 * @var string
	 */
	const ITEM_NAME = 'Tasty Recipes';

	/**
	 * Cache key used to store license check data.
	 *
	 * @var string
	 */
	const LICENSE_CHECK_CACHE_KEY = 'tasty-recipes-license-check';

	/**
	 * Key used for nonce authentication.
	 *
	 * @var string
	 */
	const NONCE_KEY = 'tasty-recipes-settings';

	/**
	 * URL to the plugin store.
	 *
	 * @var string
	 */
	const STORE_URL = 'https://www.wptasty.com';

	/**
	 * Parent page for the settings page.
	 *
	 * @var string
	 */
	const PAGE_BASE = 'options-general.php';

	/**
	 * Slug for the settings page.
	 *
	 * @var string
	 */
	const PAGE_SLUG = 'tasty-recipes';

	/**
	 * Group used for authentication settings.
	 *
	 * @var string
	 */
	const SETTINGS_GROUP_LICENSE = 'tasty-recipes-settings-license';

	/**
	 * Section used for authentication settings.
	 *
	 * @var string
	 */
	const SETTINGS_SECTION_LICENSE = 'tasty-recipes-license';

	/**
	 * Group used for card settings.
	 *
	 * @var string
	 */
	const SETTINGS_GROUP_CARD = 'tasty-recipes-settings-card';

	/**
	 * Section used for card settings.
	 *
	 * @var string
	 */
	const SETTINGS_SECTION_CARD_DESIGN = 'tasty-recipes';

	/**
	 * Available template options.
	 *
	 * @var array
	 */
	const TEMPLATE_OPTIONS = array(
		''               => 'Default',
		'bold'           => 'Bold',
		'snap'           => 'Snap',
		'fresh'          => 'Fresh',
		'simple'         => 'Simple',
		'modern-compact' => 'Modern Compact',
		'elegant'        => 'Elegant',
	);

	/**
	 * ID used for TinyMCE instance in settings.
	 *
	 * @var string
	 */
	private static $editor_id = 'tasty-recipes-settings';

	/**
	 * Registers the updater object.
	 *
	 * @return void
	 */
	public static function action_admin_init() {
		new Updater(
			self::STORE_URL,
			TASTY_RECIPES_PLUGIN_FILE,
			array(
				'version'   => TASTY_RECIPES_PLUGIN_VERSION,
				'license'   => get_option( Tasty_Recipes::LICENSE_KEY_OPTION ),
				'item_name' => self::ITEM_NAME,
				'author'    => self::ITEM_AUTHOR,
			)
		);
		if ( get_option( Tasty_Recipes::PLUGIN_ACTIVATION_OPTION, false ) ) {
			delete_option( Tasty_Recipes::PLUGIN_ACTIVATION_OPTION );
			if ( empty( Utils::get_param( 'activate-multi' ) ) ) {
				wp_safe_redirect( add_query_arg( 'tab', 'about', menu_page_url( self::PAGE_SLUG, false ) ) );
				exit;
			}
		}
	}

	/**
	 * Includes PHP and plugin versions in the user agent for update checks.
	 *
	 * @param array  $r   An array of HTTP request arguments.
	 * @param string $url The request URL.
	 *
	 * @return array
	 */
	public static function filter_http_request_args( $r, $url ) {
		if ( self::STORE_URL !== $url
			|| 'POST' !== $r['method']
			|| empty( $r['body'] )
			|| ! is_array( $r['body'] )
			|| empty( $r['body']['item_name'] )
			|| self::ITEM_NAME !== $r['body']['item_name'] ) {
			return $r;
		}

		$r['user-agent'] = rtrim( $r['user-agent'], ';' )
			. '; PHP/' . PHP_VERSION . '; '
			. self::ITEM_NAME . '/' . TASTY_RECIPES_PLUGIN_VERSION;
		return $r;
	}

	/**
	 * Registers used settings.
	 *
	 * @return void
	 */
	public static function action_admin_menu() { // phpcs:ignore SlevomatCodingStandard
		register_setting(
			self::SETTINGS_SECTION_CARD_DESIGN,
			Tasty_Recipes::CUSTOMIZATION_OPTION,
			array(
				'sanitize_callback' => array( __CLASS__, 'sanitize_customization_option' ),
			)
		);
		register_setting(
			self::SETTINGS_SECTION_CARD_DESIGN,
			Tasty_Recipes::TEMPLATE_OPTION,
			array(
				'sanitize_callback' => 'sanitize_title',
			)
		);
		register_setting(
			self::SETTINGS_GROUP_CARD,
			Tasty_Recipes::QUICK_LINKS_OPTION,
			array(
				'sanitize_callback' => array( __CLASS__, 'sanitize_quick_links' ),
			)
		);
		register_setting(
			self::SETTINGS_GROUP_CARD,
			Tasty_Recipes::QUICK_LINKS_STYLE,
			array(
				'sanitize_callback' => 'sanitize_title',
			)
		);
		register_setting(
			self::SETTINGS_GROUP_CARD,
			Tasty_Recipes::CARD_BUTTONS_OPTION,
			array(
				'sanitize_callback' => array( __CLASS__, 'sanitize_card_buttons' ),
			)
		);

		register_setting(
			self::SETTINGS_GROUP_CARD,
			Tasty_Recipes::UNIT_CONVERSION_OPTION,
			array(
				'sanitize_callback' => 'sanitize_title',
			)
		);

		register_setting(
			self::SETTINGS_GROUP_CARD,
			Tasty_Recipes::AUTOMATIC_UNIT_CONVERSION_OPTION,
			array(
				'sanitize_callback' => 'sanitize_title',
			)
		);

		register_setting(
			self::SETTINGS_GROUP_CARD,
			Tasty_Recipes::INGREDIENT_CHECKBOXES_OPTION,
			array(
				'sanitize_callback' => 'sanitize_title',
			)
		);

		register_setting(
			self::SETTINGS_GROUP_CARD,
			Tasty_Recipes::COOK_MODE_OPTION,
			array(
				'sanitize_callback' => 'sanitize_title',
			)
		);

		register_setting(
			self::SETTINGS_GROUP_CARD,
			Tasty_Recipes::COOK_MODE_HELPER_OPTION,
			array(
				'sanitize_callback' => 'sanitize_text_field',
			)
		);

		register_setting(
			self::SETTINGS_GROUP_CARD,
			Tasty_Recipes::INSTACART_OPTION,
			array(
				'sanitize_callback' => 'sanitize_title',
			)
		);

		register_setting(
			self::SETTINGS_GROUP_CARD,
			Tasty_Recipes::INSTACART_AFFILIATE_ID_OPTION,
			array(
				'sanitize_callback' => 'sanitize_title',
			)
		);

		register_setting(
			self::SETTINGS_GROUP_CARD,
			Tasty_Recipes::DISABLE_SCALING_OPTION,
			array(
				'sanitize_callback' => 'sanitize_title',
			)
		);

		register_setting(
			self::SETTINGS_GROUP_CARD,
			Tasty_Recipes::COPY_TO_CLIPBOARD_OPTION,
			array(
				'sanitize_callback' => 'sanitize_title',
			)
		);

		register_setting(
			self::SETTINGS_GROUP_CARD,
			Tasty_Recipes::DEFAULT_AUTHOR_LINK_OPTION,
			array(
				'sanitize_callback' => 'esc_url_raw',
			)
		);
		register_setting(
			self::SETTINGS_GROUP_CARD,
			Tasty_Recipes::SHAREASALE_OPTION,
			array(
				'sanitize_callback' => 'sanitize_text_field',
			)
		);
	}

	/**
	 * Renders the admin notice nag when license key isn't set or invalid.
	 *
	 * @deprecated 3.10
	 *
	 * @return void
	 */
	public static function action_admin_notices_license_key() {
		_deprecated_function( __METHOD__, '3.10' );
	}

	/**
	 * Add the callback for the plugin menu item.
	 *
	 * @param array $menu_items Admin menu items.
	 *
	 * @return array
	 */
	public static function add_menu_item( $menu_items ) {
		if ( empty( $menu_items['tasty-recipes'] ) ) {
			return $menu_items;
		}

		$menu_items['tasty-recipes']['callback'] = array( __CLASS__, 'handle_settings_page' );
		return $menu_items;
	}

	/**
	 * Registers a new custom column for Tasty Recipes.
	 *
	 * @param array $columns Existing column names.
	 *
	 * @return array
	 */
	public static function action_manage_posts_columns( $columns ) {
		$columns['tasty_recipe'] = esc_html__( 'Tasty Recipe', 'tasty-recipes' );
		return $columns;
	}

	/**
	 * Renders an 'Edit Tasty Recipe' button for the column if the post has a recipe.
	 *
	 * @param string $column_name Name of the column.
	 * @param int    $post_id     ID of the post being displayed.
	 *
	 * @return void
	 */
	public static function action_manage_posts_custom_column( $column_name, $post_id ) {
		switch ( $column_name ) {
			case 'tasty_recipe':
				$recipe_ids = Tasty_Recipes::get_recipe_ids_for_post( $post_id );
				if ( ! empty( $recipe_ids ) ) {
					$recipe_id = array_shift( $recipe_ids );
					echo '<button class="button tasty-recipes-edit-button" data-recipe-id="' . esc_attr( $recipe_id ) . '">' .
						esc_html__( 'Edit Tasty Recipe', 'tasty-recipes' ) .
						'</button>';
				}
				break;
		}
	}

	/**
	 * Renders the 'Edit Tasty Recipe' button on Quick Edit.
	 *
	 * @param string $column_name Name of the column.
	 *
	 * @return void
	 */
	public static function action_quick_edit_custom_box( $column_name ) {
		switch ( $column_name ) {
			case 'tasty_recipe':
				echo '<button class="button tasty-recipes-edit-button" data-recipe-id="">' . esc_html__( 'Edit Tasty Recipe', 'tasty-recipes' ) . '</button>';
				break;
		}
	}

	/**
	 * Hides the 'Tasty Recipe' column by default.
	 *
	 * @param array $columns Existing default hidden columns.
	 *
	 * @return array
	 */
	public static function filter_hidden_columns( $columns ) {
		$columns[] = 'tasty_recipe';
		return $columns;
	}

	/**
	 * Activates the license when the option is updated.
	 *
	 * @deprecated 3.10
	 *
	 * @return void
	 */
	public static function action_update_option_register_license() {
		_deprecated_function( __METHOD__, '3.10' );
	}

	/**
	 * Clears the license and version check cache when license key is updated.
	 *
	 * @return void
	 */
	public static function action_update_option_clear_transient() {
		delete_transient( self::LICENSE_CHECK_CACHE_KEY );
		$cache_key = md5( 'edd_plugin_' . sanitize_key( plugin_basename( TASTY_RECIPES_PLUGIN_FILE ) ) . '_version_info' );
		delete_site_transient( $cache_key );
		// phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions.serialize_serialize
		$cache_key = 'edd_api_request_' . substr( md5( serialize( basename( TASTY_RECIPES_PLUGIN_FILE, '.php' ) ) ), 0, 15 );
		delete_site_transient( $cache_key );
		delete_site_transient( 'update_plugins' );
	}

	/**
	 * Adds 'Settings' and 'Remove license' links to plugin settings.
	 *
	 * @param array $links Existing plugin action links.
	 *
	 * @return array
	 */
	public static function filter_plugin_action_links( $links ) {
		$links['tr_settings']        = '<a href="' . esc_url( add_query_arg( 'tab', 'settings', menu_page_url( self::PAGE_SLUG, false ) ) ) . '">' .
			esc_html__( 'Settings', 'tasty-recipes' ) .
			'</a>';
		$links['tr_license_manager'] = '<a href="' . esc_url( Url::get_main_admin_url() ) . '">' . esc_html__( 'License manager', 'tasty-recipes' ) . '</a>';
		return $links;
	}

	/**
	 * Handles a request to remove the license key.
	 *
	 * @deprecated 3.10
	 *
	 * @return void
	 */
	public static function handle_wp_ajax_remove_license_key() {
		_deprecated_function( __METHOD__, '3.10' );

		wp_safe_redirect( admin_url( 'plugins.php' ) );
		exit;
	}

	/**
	 * Handles an AJAX request to preview a recipe template.
	 *
	 * @return void
	 */
	public static function handle_wp_ajax_preview_recipe_card() { // phpcs:ignore SlevomatCodingStandard
		global $editor_styles;
		$nonce = Utils::request_param( 'nonce', 'sanitize_key' );
		if ( empty( $nonce ) || ! current_user_can( self::CAPABILITY ) || ! wp_verify_nonce( $nonce, self::NONCE_KEY ) ) {
			wp_die( esc_html__( "Sorry, you don't have permission to do this.", 'tasty-recipes' ) );
		}
		header( 'Content-Type: text/html' );
		$wrapper_classes = array(
			'tasty-recipes',
			'tasty-recipes-display',
			'tasty-recipes-has-image',
		);

		$shareasale = get_option( Tasty_Recipes::SHAREASALE_OPTION );
		if ( $shareasale ) {
			$wrapper_classes[] = 'tasty-recipes-has-plug';
		}

		$get_template               = Utils::get_param( 'template', 'sanitize_key' );
		$get_star_ratings_style     = Utils::get_param( 'star_ratings_style', 'sanitize_key' );
		$get_nutrifox_display_style = Utils::get_param( 'nutrifox_display_style', 'sanitize_key' );
		$get_footer_social_platform = Utils::get_param( 'footer_social_platform', 'sanitize_key' );

		$custom_design          = ! empty( $get_template ) && array_key_exists( $get_template, self::TEMPLATE_OPTIONS ) ? $get_template : '';
		$star_ratings_style     = ! empty( $get_star_ratings_style ) && in_array( $get_star_ratings_style, array( 'solid', 'outline' ), true ) ? $get_star_ratings_style : null;
		$show_nutrifox          = ! empty( $get_nutrifox_display_style ) && in_array( $get_nutrifox_display_style, array( 'label', 'card' ), true );
		$nutrifox_display_style = $show_nutrifox ? $get_nutrifox_display_style : null;
		$show_socials           = ! empty( $get_footer_social_platform ) && in_array( $get_footer_social_platform, array( 'instagram', 'pinterest', 'facebook' ), true );
		$footer_social_platform = $show_socials ? $get_footer_social_platform : '';

		$settings = Tasty_Recipes::get_filtered_customization_settings();

		$styles  = Shortcodes::get_styles_as_string( $custom_design );
		$styles .= Utils::get_contents( dirname( TASTY_RECIPES_PLUGIN_FILE ) . '/assets/dist/ratings.css' );
		$styles .= Utils::get_contents( dirname( TASTY_RECIPES_PLUGIN_FILE ) . '/assets/dist/recipe-card-preview.css' );

		$custom_path = dirname( TASTY_RECIPES_PLUGIN_FILE ) . '/templates/designs/' . $custom_design . '/tasty-recipes.php';
		if ( $custom_design && file_exists( $custom_path ) ) {
			$template = $custom_path;
		} else {
			$template = 'recipe/tasty-recipes';
		}

		$stylesheet = get_option( 'stylesheet' );
		// Clean up Feast directory names.
		$stylesheet  = preg_replace( '#-v[\d]+$#', '', $stylesheet );
		$compat_file = dirname( TASTY_RECIPES_PLUGIN_FILE ) . '/assets/css/theme-compat-card-previews/' . $stylesheet . '.css';
		if ( file_exists( $compat_file ) ) {
			$styles .= Utils::get_contents( $compat_file );
		}

		$recipe      = new \stdClass();
		$recipe_json = array();
		$ingredients = <<<EOT
<ul>
	<li><span data-amount="3/4" data-unit="cup">3/4 cup</span> melted coconut oil</li>
	<li><span data-amount="1/2" data-unit="cup">1/2 cup</span> cocoa powder</li>
	<li><span data-amount="2" data-unit="tablespoon">2 tbsp.</span> natural sweetener (agave or maple syrup)</li>
	<li><span data-amount="3/4" data-unit="cup">3/4 cup</span> almond butter</li>
	<li><span data-amount="" data-unit=""></span>pinch of sea salt, plus more for topping</li>
</ul>
EOT;
		$ingredients = apply_filters( 'tasty_recipes_the_content', $ingredients, 'ingredients' );

		// phpcs:disable SlevomatCodingStandard.Files.LineLength.LineTooLong
		$instructions = <<<EOT
<ol>
<li id="instruction-step-1">Whisk the coconut oil, cocoa powder, sweetener, and a pinch of salt.</li>
<li id="instruction-step-2">Fill a regular size muffin tin with paper liners. Pour a small amount of the cocoa mixture (1-2 tablespoons) into the paper cups. Drop a small spoonful of the almond butter (2-3 teaspoons) into the center of each cup. Divide remaining chocolate amongst the cups.</li>
<li id="instruction-step-3">If almond butter is sticking up, just gently press it down so each cup has a smooth top layer. Sprinkle each almond butter cup with a pinch of coarse sea salt. Freeze for one hour or until solid. YUM TOWN.</li>
</ol>
EOT;
		$notes        = <<<EOT
<p>These are very adaptable – I’ve used as much as 3/4 cup cocoa, and as little as 2 tablespoons agave. It just depends on how sweet / dark you want them to be. I’ve also used peanut butter which is (obviously) delicious!</p>
EOT;
		// phpcs:enable

		if ( 'card' === $nutrifox_display_style ) {
			$nutrifox_id           = '';
			$recipe_nutrition      = array(
				'serving_size' =>
				array(
					'label' => 'Serving Size',
					'value' => '<span data-tasty-recipes-customization="body-color.color" class="tasty-recipes-serving-size">12</span>',
				),
				'calories'     =>
				array(
					'label' => 'Calories',
					'value' => '<span data-tasty-recipes-customization="body-color.color" class="tasty-recipes-calories">238</span>',
				),
				'sugar'        =>
				array(
					'label' => 'Sugar',
					'value' => '<span data-tasty-recipes-customization="body-color.color" class="tasty-recipes-sugar">3.8g</span>',
				),
				'sodium'       =>
				array(
					'label' => 'Sodium',
					'value' => '<span data-tasty-recipes-customization="body-color.color" class="tasty-recipes-sodium">99.3mg</span>',
				),
				'fat'          =>
				array(
					'label' => 'Fat',
					'value' => '<span data-tasty-recipes-customization="body-color.color" class="tasty-recipes-fat">22.6g</span>',
				),
				'fiber'        =>
				array(
					'label' => 'Fiber',
					'value' => '<span data-tasty-recipes-customization="body-color.color" class="tasty-recipes-fiber">2.7g</span>',
				),
				'protein'      =>
				array(
					'label' => 'Protein',
					'value' => '<span data-tasty-recipes-customization="body-color.color" class="tasty-recipes-protein">3.9g</span>',
				),
			);
			$recipe_nutrifox_embed = '';
		} else {
			$recipe_nutrition       = array();
			$nutrifox_id            = 26460;
			$nutrifox_iframe_url    = sprintf(
				'%s/embed/label/%d',
				Tasty_Recipes::nutrifox_url(),
				$nutrifox_id
			);
			$nutrifox_resize_script = Utils::get_contents( dirname( __DIR__ ) . '/assets/js/nutrifox-resize.js' );

			$recipe_nutrifox_embed = <<<EOT
<script data-cfasync="false">
{$nutrifox_resize_script}
</script>
<iframe title="nutritional information" id="nutrifox-label-{$nutrifox_id}" src="{$nutrifox_iframe_url}" style="width:100%;border-width:0;"></iframe>
EOT;
		}

		$template_vars = array(
			'recipe'                        => $recipe,
			'recipe_styles'                 => '',
			'recipe_scripts'                => '',
			'recipe_json'                   => $recipe_json,
			'recipe_title'                  => 'Almond Butter Cups',
			'recipe_image'                  => '<img width="183" height="183" ' .
				'src="' . esc_url( plugins_url( 'assets/images/Almond-Butter-Cups-Recipe-370x370.jpg', __DIR__ ) ) . '" ' .
				'class="attachment-thumbnail size-thumbnail" alt="Chocolate Chip Cookies on parchment paper." ' .
				'loading="lazy" data-pin-nopin="true">',
			'recipe_rating_icons'           => Ratings::get_rendered_rating(
				4.6,
				'star-color.color',
				$star_ratings_style
			),
			'recipe_rating_label'           => '<span data-tasty-recipes-customization="detail-label-color.color" class="rating-label">' .
				'<span class="average">4.6</span> from <span class="count">1376</span> reviews</span>',
			'recipe_author_name'            => '',
			'recipe_details'                => array(
				'author'     =>
				array(
					'label' => 'Author',
					'value' => '<a data-tasty-recipes-customization="detail-value-color.color" class="tasty-recipes-author-name" ' .
						'href="https://pinchofyum.com/about">Pinch of Yum</a>',
					'class' => 'author',
				),
				'prep_time'  =>
				array(
					'label' => 'Prep Time',
					'value' => '<span data-tasty-recipes-customization="detail-value-color.color" class="tasty-recipes-prep-time">10 mins</span>',
					'class' => 'prep-time',
				),
				'cook_time'  =>
				array(
					'label' => 'Cook Time',
					'value' => '<span data-tasty-recipes-customization="detail-value-color.color" class="tasty-recipes-cook-time">1 hour (for freezing)</span>',
					'class' => 'cook-time',
				),
				'total_time' =>
				array(
					'label' => 'Total Time',
					'value' => '<span data-tasty-recipes-customization="detail-value-color.color" class="tasty-recipes-total-time">1 hour, 10 minutes</span>',
					'class' => 'total-time',
				),
				'yield'      =>
				array(
					'label' => 'Yield',
					'value' => '<span data-tasty-recipes-customization="detail-value-color.color" class="tasty-recipes-yield">' .
						'<span data-amount="12">12</span> almond butter cups' .
						'<span class="tasty-recipes-yield-scale"><span data-amount="1">1</span>x</span></span>',
					'class' => 'yield',
				),
				'category'   =>
				array(
					'label' => 'Category',
					'value' => '<span data-tasty-recipes-customization="detail-value-color.color" class="tasty-recipes-category">Dessert</span>',
					'class' => 'category',
				),
				'method'     =>
				array(
					'label' => 'Method',
					'value' => '<span data-tasty-recipes-customization="detail-value-color.color" class="tasty-recipes-method">No bake</span>',
					'class' => 'method',
				),
				'cuisine'    =>
				array(
					'label' => 'Cuisine',
					'value' => '<span data-tasty-recipes-customization="detail-value-color.color" class="tasty-recipes-cuisine">American</span>',
					'class' => 'cuisine',
				),
			),
			'recipe_description'            => '<p>Almond Butter Cups: made with five ingredients and no refined sugar. So creamy, rich, and yummy!</p>',
			'recipe_ingredients'            => $ingredients,
			'recipe_instructions_has_video' => false,
			'recipe_scalable'               => '<button class="tasty-recipes-scale-button tasty-recipes-scale-button-active" data-amount="1">1x</button>' .
				'<button class="tasty-recipes-scale-button" data-amount="2">2x</button>' .
				'<button class="tasty-recipes-scale-button" data-amount="3">3x</button>',
			'recipe_instructions'           => $instructions,
			'recipe_keywords'               => '',
			'recipe_notes'                  => $notes,
			'recipe_nutrifox_id'            => $nutrifox_id,
			'recipe_nutrifox_embed'         => $recipe_nutrifox_embed,
			'recipe_video_embed'            => '',
			'recipe_nutrition'              => $recipe_nutrition,
			'recipe_hidden_nutrition'       => array(),
			'copy_ingredients'              => get_option( Tasty_Recipes::COPY_TO_CLIPBOARD_OPTION )
				? '<button aria-label="' . esc_attr__( 'Copy ingredients to clipboard', 'tasty-recipes' ) . '" ' .
				'class="tasty-recipes-copy-button" id="copy-ingredients" data-text="' . esc_attr__( 'Copy ingredients', 'tasty-recipes' ) . '">' .
				Utils::get_contents( dirname( __DIR__ ) . '/assets/images/icons/icon-copy-to-clipboard.svg' ) .
				'</button>'
				: false,
			'first_button'                  => Shortcodes::get_card_button( $recipe, 'first', $custom_design ),
			'second_button'                 => Shortcodes::get_card_button( $recipe, 'second', $custom_design ),
			'instagram_handle'              => get_option( Tasty_Recipes::INSTAGRAM_HANDLE_OPTION ),
			'instagram_hashtag'             => get_option( Tasty_Recipes::INSTAGRAM_HASHTAG_OPTION ),
			'footer_social_platform'        => $footer_social_platform,
			'footer_heading'                => $settings['footer_heading'],
			'footer_description'            => $settings['footer_description'],
		);

		$stylesheets = '';
		if ( $editor_styles && current_theme_supports( 'editor-styles' ) ) {
			foreach ( $editor_styles as $style ) {
				if ( preg_match( '~^(https?:)?//~', $style ) ) {
					// phpcs:ignore WordPress.WP.EnqueuedResources.NonEnqueuedStylesheet
					$stylesheets .= '<link rel="stylesheet" href="' . esc_url( $style ) . '">' . PHP_EOL;
				} else {
					 // phpcs:ignore WordPress.WP.EnqueuedResources.NonEnqueuedStylesheet
					$stylesheets .= '<link rel="stylesheet" href="' . esc_url( get_theme_file_uri( $style ) ) . '">' . PHP_EOL;
				}
			}
		}

		$template_vars['recipe_styles'] = '<style>' . $styles . PHP_EOL . '</style>' . PHP_EOL;

		$ret            = '<!DOCTYPE html><html><head>' .
			$stylesheets . self::load_enqueued_assets() .
			'</head><body>';
		$ret           .= '<div class="' . esc_attr( implode( ' ', $wrapper_classes ) ) . '"';
		$customizations = Shortcodes::get_card_container_customizations( $custom_design );
		if ( $customizations ) {
			$ret .= ' data-tasty-recipes-customization="' . esc_attr( $customizations ) . '"';
		}
		$ret .= '>' . PHP_EOL;
		$ret .= Tasty_Recipes::get_template_part( $template, $template_vars );
		$ret .= '</div>';
		$ret .= '<script>' . PHP_EOL . Utils::get_contents( dirname( __DIR__ ) . '/assets/dist/settings-recipe-card-preview.build.js' ) . PHP_EOL . '</script>';
		$ret .= '</body></html>';
		echo apply_filters( 'tasty_recipes_recipe_card_output', $ret, 'admin', compact( 'recipe' ) ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		exit;
	}

	/**
	 * Get the html for enqueued assets for the recipe card preview when doing ajax.
	 *
	 * @since 3.12.3
	 *
	 * @return string
	 */
	private static function load_enqueued_assets() {
		ob_start();
		wp_print_styles();
		wp_print_scripts();
		$assets = ob_get_clean();
		if ( ! $assets ) {
			$assets = '';
		}

		return $assets;
	}

	/**
	 * Handles an AJAX request to get the number of matching recipes to convert.
	 *
	 * @return void
	 */
	public static function handle_wp_ajax_get_count() {
		$nonce = Utils::get_param( 'nonce', 'sanitize_key' );
		if ( empty( $nonce ) || ! current_user_can( self::CAPABILITY ) || ! wp_verify_nonce( $nonce, self::NONCE_KEY ) ) {
			Utils::send_json_error( __( 'Sorry, you don\'t have permission to do this.', 'tasty-recipes' ) );
		}

		$class      = false;
		$converters = Tasty_Recipes::get_converters();
		$get_type   = Utils::get_param( 'type' );

		if ( ! empty( $get_type ) && isset( $converters[ $get_type ] ) ) {
			$class = $converters[ $get_type ]['class'];
		} else {
			Utils::send_json_error( __( 'Couldn\'t count recipes. Please contact support for assistance.', 'tasty-recipes' ) );
		}

		wp_send_json_success(
			array(
				'count' => (int) $class::get_count(),
			)
		);
	}

	/**
	 * Handles an AJAX request to batch convert some recipes.
	 *
	 * @return void
	 */
	public static function handle_wp_ajax_convert() {
		$nonce = Utils::get_param( 'nonce', 'sanitize_key' );
		if ( empty( $nonce ) || ! current_user_can( self::CAPABILITY ) || ! wp_verify_nonce( $nonce, self::NONCE_KEY ) ) {
			Utils::send_json_error( __( 'Sorry, you don\'t have permission to do this.', 'tasty-recipes' ) );
		}

		$class      = false;
		$converters = Tasty_Recipes::get_converters();
		$get_type   = Utils::get_param( 'type' );

		if ( ! empty( $get_type ) && isset( $converters[ $get_type ] ) ) {
			$class = $converters[ $get_type ]['class'];
		} else {
			Utils::send_json_error(
				__( 'Couldn\'t convert recipes. Please contact support for assistance.', 'tasty-recipes' )
			);
		}

		$post_ids  = $class::get_post_ids();
		$converted = 0;
		foreach ( $post_ids as $post_id ) {
			$type = 'shortcode';
			if ( function_exists( 'has_blocks' )
				&& has_blocks( $post_id ) ) {
				$type = 'block';
			}
			$recipe = $class::convert_post( $post_id, $type );
			if ( $recipe ) {
				++$converted;
			} else {
				Utils::send_json_error(
					sprintf(
						/* translators: %1$s: post title, %2$d: post ID */
						__( 'Couldn\'t convert recipe in \'%1$s\' (post %2$d). Please contact support for assistance.', 'tasty-recipes' ),
						get_the_title( $post_id ),
						$post_id
					)
				);
			}
		}

		$after_count = $class::get_count();
		wp_send_json_success(
			array(
				'converted' => $converted,
				'count'     => (int) $after_count,
			)
		);
	}

	/**
	 * Renders in the admin header.
	 *
	 * @deprecated 3.10
	 *
	 * @return void
	 */
	public static function action_in_admin_header() {
		_deprecated_function( __METHOD__, '3.10' );
	}

	/**
	 * Renders the Tasty Recipes settings page.
	 *
	 * @return void
	 */
	public static function handle_settings_page() { // phpcs:ignore SlevomatCodingStandard
		add_settings_section(
			self::SETTINGS_SECTION_CARD_DESIGN,
			'',
			'__return_false',
			self::PAGE_SLUG
		);
		add_settings_field(
			Tasty_Recipes::QUICK_LINKS_OPTION,
			__( 'Quick Links', 'tasty-recipes' ),
			array( __CLASS__, 'render_multiselect_checkboxes_field' ),
			self::PAGE_SLUG,
			self::SETTINGS_SECTION_CARD_DESIGN,
			array(
				'name'           => Tasty_Recipes::QUICK_LINKS_OPTION,
				'description'    => __( 'Enable or disable quick links below the post title.', 'tasty-recipes' ),
				'default_values' => array(
					'jump',
					'print',
				),
				'options'        => array(
					'jump'   => __( '\'Jump to Recipe\' link', 'tasty-recipes' ),
					'print'  => __( '\'Print Recipe\' link', 'tasty-recipes' ),
					'rating' => __( '\'Leave a Review\' link', 'tasty-recipes' ),
				),
			)
		);
		add_settings_field(
			Tasty_Recipes::QUICK_LINKS_STYLE,
			__( 'Quick Links Style', 'tasty-recipes' ),
			array( __CLASS__, 'render_select_field' ),
			self::PAGE_SLUG,
			self::SETTINGS_SECTION_CARD_DESIGN,
			array(
				'name'          => Tasty_Recipes::QUICK_LINKS_STYLE,
				'options'       => array(
					'links'   => __( 'Links', 'tasty-recipes' ),
					'buttons' => __( 'Buttons', 'tasty-recipes' ),
				),
				'default_value' => 'links',
			)
		);
		add_settings_field(
			Tasty_Recipes::CARD_BUTTONS_OPTION,
			__( 'Recipe Card Buttons', 'tasty-recipes' ),
			array( __CLASS__, 'render_card_buttons_field' ),
			self::PAGE_SLUG,
			self::SETTINGS_SECTION_CARD_DESIGN
		);
		add_settings_field(
			Tasty_Recipes::UNIT_CONVERSION_OPTION,
			__( 'Unit Conversion', 'tasty-recipes' ),
			array( __CLASS__, 'render_select_field' ),
			self::PAGE_SLUG,
			self::SETTINGS_SECTION_CARD_DESIGN,
			array(
				'name'              => Tasty_Recipes::UNIT_CONVERSION_OPTION,
				'description'       => __( 'Allow visitors to convert between US Customary and Metric.', 'tasty-recipes' ),
				'options'           => array(
					false => __( 'Off', 'tasty-recipes' ),
					true  => __( 'On', 'tasty-recipes' ),
				),
				'after_description' => function () {
					$value = get_option( Tasty_Recipes::AUTOMATIC_UNIT_CONVERSION_OPTION, false );
					echo '<p class="description tasty-recipes-automatic-unit-conversion-container" style="display:none;">';
					echo '<label>';
					echo '<input type="checkbox" name="' . esc_attr( Tasty_Recipes::AUTOMATIC_UNIT_CONVERSION_OPTION ) . '" ' .
						checked( (bool) $value, true, false ) .
						'>';
					esc_html_e( 'Automatically apply unit conversion to existing recipes.', 'tasty-recipes' );
					echo '</label>';
					echo '</p>';
					// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
					echo <<<EOT
<script>
(function(){
	var select = document.querySelector('select[name="tasty_recipes_unit_conversion"]');
	var container = document.querySelector('.tasty-recipes-automatic-unit-conversion-container');
	if (1 === parseInt(select.value)) {
		container.style.display = null;
	}
	select.addEventListener('change', function() {
		if (1 === parseInt(select.value)) {
			container.style.display = null;
		} else {
			container.style.display = 'none';
		}
	});
}())
</script>
EOT;
				},
			)
		);
		add_settings_field(
			Tasty_Recipes::INGREDIENT_CHECKBOXES_OPTION,
			__( 'Ingredient Checkboxes', 'tasty-recipes' ),
			array( __CLASS__, 'render_select_field' ),
			self::PAGE_SLUG,
			self::SETTINGS_SECTION_CARD_DESIGN,
			array(
				'name'        => Tasty_Recipes::INGREDIENT_CHECKBOXES_OPTION,
				'description' => __( 'Allow visitors to check off recipe ingredients.', 'tasty-recipes' ),
				'options'     => array(
					false => __( 'Off', 'tasty-recipes' ),
					true  => __( 'On', 'tasty-recipes' ),
				),
			)
		);

		add_settings_field(
			Tasty_Recipes::COOK_MODE_OPTION,
			__( 'Enable Cook Mode', 'tasty-recipes' ),
			array( __CLASS__, 'render_select_field' ),
			self::PAGE_SLUG,
			self::SETTINGS_SECTION_CARD_DESIGN,
			array(
				'name'              => Tasty_Recipes::COOK_MODE_OPTION,
				'description'       => __( 'Show Cook Mode toggle to let visitors prevent sleep mode for supported devices.', 'tasty-recipes' ),
				'options'           => array(
					true  => __( 'On', 'tasty-recipes' ),
					false => __( 'Off', 'tasty-recipes' ),
				),
				'disabled'          => ! self::can_enable_cook_mode(),
				'default_value'     => self::can_enable_cook_mode(),
				'forced_value'      => self::can_enable_cook_mode() ? null : false,
				'after_description' => array( __CLASS__, 'show_ssl_error' ),
			)
		);

		add_settings_field(
			Tasty_Recipes::COOK_MODE_HELPER_OPTION,
			__( 'Cook Mode description', 'tasty-recipes' ),
			array( __CLASS__, 'render_input_field' ),
			self::PAGE_SLUG,
			self::SETTINGS_SECTION_CARD_DESIGN,
			array(
				'name'          => Tasty_Recipes::COOK_MODE_HELPER_OPTION,
				'default_value' => esc_html__( 'Prevent your screen from going dark', 'tasty-recipes' ),
			)
		);

		add_settings_field(
			Tasty_Recipes::DISABLE_SCALING_OPTION,
			__( 'Scaling', 'tasty-recipes' ),
			array( __CLASS__, 'render_select_field' ),
			self::PAGE_SLUG,
			self::SETTINGS_SECTION_CARD_DESIGN,
			array(
				'name'        => Tasty_Recipes::DISABLE_SCALING_OPTION,
				'description' => __( 'Allow visitors to scale the recipe 1x, 2x, or 3x.', 'tasty-recipes' ),
				'options'     => array(
					true  => __( 'Off', 'tasty-recipes' ),
					false => __( 'On', 'tasty-recipes' ),
				),
			)
		);
		add_settings_field(
			Tasty_Recipes::COPY_TO_CLIPBOARD_OPTION,
			__( 'Copy to Clipboard', 'tasty-recipes' ),
			array( __CLASS__, 'render_select_field' ),
			self::PAGE_SLUG,
			self::SETTINGS_SECTION_CARD_DESIGN,
			array(
				'name'        => Tasty_Recipes::COPY_TO_CLIPBOARD_OPTION,
				'description' => __( 'Allow visitors to copy ingredients to clipboard.', 'tasty-recipes' ),
				'options'     => array(
					false => __( 'Off', 'tasty-recipes' ),
					true  => __( 'On', 'tasty-recipes' ),
				),
			)
		);
		add_settings_field(
			Tasty_Recipes::DEFAULT_AUTHOR_LINK_OPTION,
			__( 'Default Author Link', 'tasty-recipes' ),
			array( __CLASS__, 'render_input_field' ),
			self::PAGE_SLUG,
			self::SETTINGS_SECTION_CARD_DESIGN,
			array(
				'name' => Tasty_Recipes::DEFAULT_AUTHOR_LINK_OPTION,
			)
		);

		add_settings_field(
			Tasty_Recipes::INSTACART_OPTION,
			__( 'Add an Instacart button', 'tasty-recipes' ),
			array( __CLASS__, 'render_select_field' ),
			self::PAGE_SLUG,
			self::SETTINGS_SECTION_CARD_DESIGN,
			array(
				'name'          => Tasty_Recipes::INSTACART_OPTION,
				'description'   => sprintf(
					// translators: %1$s opening anchor tag, %2$s closing tag.
					esc_html__( 'By turning this on, you agree to the %1$sInstacart terms of service%2$s, available in the US only at the moment.', 'tasty-recipes' ),
					'<a target="_blank" rel="noopener" href="https://affiliate.instacart.com/terms">',
					'</a>'
				),
				'options'       => array(
					true  => __( 'On', 'tasty-recipes' ),
					false => __( 'Off', 'tasty-recipes' ),
				),
				'default_value' => false,
			)
		);

		add_settings_field(
			Tasty_Recipes::INSTACART_AFFILIATE_ID_OPTION,
			__( 'Instacart Tastemakers Affiliate ID', 'tasty-recipes' ),
			array( __CLASS__, 'render_input_field' ),
			self::PAGE_SLUG,
			self::SETTINGS_SECTION_CARD_DESIGN,
			array(
				'name'        => Tasty_Recipes::INSTACART_AFFILIATE_ID_OPTION,
				'description' => sprintf(
					// translators: %1$s opening anchor tag, %2$s closing tag.
					__( '%1$sGet commissions%2$s when visitors order ingredients.', 'tasty-recipes' ),
					'<a target="_blank" rel="noopener" href="https://www.wptasty.com/go/instacart">',
					'</a>'
				),
			)
		);

		add_settings_field(
			Tasty_Recipes::SHAREASALE_OPTION,
			__( 'ShareASale Affiliate ID', 'tasty-recipes' ),
			array( __CLASS__, 'render_input_field' ),
			self::PAGE_SLUG,
			self::SETTINGS_SECTION_CARD_DESIGN,
			array(
				'name'        => Tasty_Recipes::SHAREASALE_OPTION,
				'description' => sprintf(
					// translators: %1$s opening link tag, %2$s closing tag, %3$s opening link tag, %4$s closing tag.
					esc_html__( '%1$sApply for the affiliate program%2$s, or %3$sfind your affiliate ID%4$s.', 'tasty-recipes' ),
					'<a href="https://www.wptasty.com/affiliate" target="_blank">',
					'</a>',
					'<a href="https://www.wptasty.com/affiliate-id" target="_blank">',
					'</a>'
				),
			)
		);

		self::show_settings();
		if ( ! did_action( 'admin_footer' ) && ! doing_action( 'admin_footer' ) ) {
			add_action( 'admin_footer', array( __CLASS__, 'action_admin_footer_render_template' ) );
		}
	}

	/**
	 * Set the tab links and include the settings template.
	 *
	 * @since 3.11.1
	 *
	 * @return void
	 */
	private static function show_settings() {
		$active_tab = Utils::sanitize_get_key( 'tab', 'design' );
		$base_url   = menu_page_url( self::PAGE_SLUG, false );
		$debug      = Utils::sanitize_get_key( 'debug' );
		$tabs       = array();

		$add_link = array(
			'design'     => __( 'Design', 'tasty-recipes' ),
			'settings'   => __( 'Settings', 'tasty-recipes' ),
			'converters' => __( 'Converters', 'tasty-recipes' ),
		);

		if ( ! empty( $debug ) || 'debug' === $active_tab ) {
			$add_link['debug'] = __( 'Debug', 'tasty-recipes' );
		}

		$add_link['about'] = __( 'About', 'tasty-recipes' );

		foreach ( $add_link as $key => $title ) {
			$tabs[] = array(
				'title'  => $title,
				'url'    => add_query_arg( array( 'tab' => $key ), $base_url ),
				'active' => $key === $active_tab,
			);
		}

		Tasty_Recipes::echo_template_part( 'settings', compact( 'tabs', 'active_tab' ) );
	}

	/**
	 * Sets the TinyMCE editor buttons for our editor instance.
	 *
	 * @param array  $buttons   Existing TinyMCE buttons.
	 * @param string $editor_id ID for the editor instance.
	 *
	 * @return array
	 */
	public static function filter_teeny_mce_buttons( $buttons, $editor_id ) {
		if ( self::$editor_id !== $editor_id ) {
			return $buttons;
		}
		return array( 'bold', 'italic', 'underline', 'bullist', 'numlist', 'link', 'unlink', 'removeformat' );
	}

	/**
	 * Prints the recipe editor modal template, but only once.
	 *
	 * @return void
	 */
	public static function action_admin_footer_render_template() {
		static $rendered_once;
		if ( isset( $rendered_once ) ) {
			return;
		}
		$rendered_once = true;
		// We don't actually care about rendering the textarea; we just want
		// the settings registered in an accessible way.
		ob_start();
		wp_editor(
			'',
			self::$editor_id,
			array(
				'teeny'   => true,
				'wpautop' => true,
			)
		);
		ob_get_clean();
	}

	/**
	 * Render an input field.
	 *
	 * @param array $args Configuration arguments used by the input field.
	 *
	 * @return void
	 */
	public static function render_input_field( $args ) {
		$defaults = array(
			'type'          => 'text',
			'name'          => '',
			'default_value' => '',
		);
		$args     = array_merge( $defaults, $args );
		if ( empty( $args['name'] ) ) {
			return;
		}
		$value = get_option( $args['name'], $args['default_value'] );
		?>
		<input type="<?php echo esc_attr( $args['type'] ); ?>"
			name="<?php echo esc_attr( $args['name'] ); ?>"
			value="<?php echo esc_attr( $value ); ?>"
			class="regular-text"
		>
		<?php if ( ! empty( $args['description'] ) ) : ?>
			<p class="description"><?php echo wp_kses_post( $args['description'] ); ?></p>
			<?php
		endif;
	}

	/**
	 * Sanitize quick links option field.
	 *
	 * @param array $item Array of items being selected.
	 *
	 * @return array
	 */
	public static function sanitize_quick_links( $item ) {
		return Utils::sanitize_item( $item, 'sanitize_key' );
	}

	/**
	 * Sanitizes the card buttons option.
	 *
	 * @param array $original Original card button option value.
	 *
	 * @return array
	 */
	public static function sanitize_card_buttons( $original ) {
		$sanitized = array();
		foreach ( array( 'first', 'second' ) as $position ) {
			if ( in_array( $original[ $position ], array( '', 'print', 'pin', 'mediavine', 'slickstream' ), true ) ) {
				$sanitized[ $position ] = $original[ $position ];
			}
		}
		return $sanitized;
	}

	/**
	 * Sanitizes the customization option to make sure there are only expected values.
	 *
	 * @param array $original Original customization option value.
	 *
	 * @return array
	 */
	public static function sanitize_customization_option( $original ) {
		$original  = stripslashes_deep( (array) $original );
		$sanitized = array();
		$allowed   = array(
			'primary_color'            => array( __CLASS__, 'sanitize_color' ),
			'secondary_color'          => array( __CLASS__, 'sanitize_color' ),
			'icon_color'               => array( __CLASS__, 'sanitize_color' ),
			'button_color'             => array( __CLASS__, 'sanitize_color' ),
			'button_text_color'        => array( __CLASS__, 'sanitize_color' ),
			'detail_label_color'       => array( __CLASS__, 'sanitize_color' ),
			'detail_value_color'       => array( __CLASS__, 'sanitize_color' ),
			'h2_color'                 => array( __CLASS__, 'sanitize_color' ),
			'h2_transform'             => array(
				'uppercase',
				'initial',
				'lowercase',
			),
			'h3_color'                 => array( __CLASS__, 'sanitize_color' ),
			'h3_transform'             => array(
				'uppercase',
				'initial',
				'lowercase',
			),
			'body_color'               => array( __CLASS__, 'sanitize_color' ),
			'star_ratings_style'       => array(
				'solid',
				'outline',
			),
			'star_color'               => array( __CLASS__, 'sanitize_color' ),
			'nutrifox_display_style'   => array(
				'label',
				'card',
			),
			'footer_social_platform'   => array(
				'instagram',
				'pinterest',
				'facebook',
			),
			'footer_icon_color'        => array( __CLASS__, 'sanitize_color' ),
			'footer_heading'           => 'sanitize_text_field',
			'footer_heading_color'     => array( __CLASS__, 'sanitize_color' ),
			'footer_description'       => 'wp_filter_post_kses',
			'footer_description_color' => array( __CLASS__, 'sanitize_color' ),
		);
		foreach ( $original as $key => $value ) {
			if ( ! isset( $allowed[ $key ] ) ) {
				continue;
			}

			if ( is_callable( $allowed[ $key ] ) ) {
				$sanitized[ $key ] = $allowed[ $key ]( $value );
			} elseif ( is_array( $allowed[ $key ] ) ) {
				if ( in_array( $value, $allowed[ $key ], true ) ) {
					$sanitized[ $key ] = $value;
				} else {
					$sanitized[ $key ] = '';
				}
			}
		}
		if ( ! empty( $sanitized['footer_description'] ) ) {
			$sanitized['footer_description'] = stripslashes( wpautop( $sanitized['footer_description'] ) );
		}
		return $sanitized;
	}

	/**
	 * Sanitizes a color field.
	 *
	 * @param string $color Existing color value.
	 *
	 * @return string
	 */
	private static function sanitize_color( $color ) {
		if ( empty( $color ) || is_array( $color ) ) {
			return '';
		}

		if ( false === strpos( $color, 'rgba' ) ) {
			return self::sanitize_hex_color( $color );
		}

		$color = str_replace( ' ', '', $color );
		sscanf( $color, 'rgba(%d,%d,%d,%f)', $red, $green, $blue, $alpha );
		return 'rgba(' . $red . ',' . $green . ',' . $blue . ',' . $alpha . ')';
	}

	/**
	 * This is a modified version of the WordPress core function.
	 * This one also allows for 8 digit hex colors.
	 *
	 * @since 3.12.4
	 *
	 * @param string $color The color to sanitize.
	 *
	 * @return string The sanitized color.
	 */
	private static function sanitize_hex_color( $color ) {
		// 3, 6, or 8 hex digits, or the empty string.
		if ( preg_match( '|^#([A-Fa-f0-9]{3}){1,2}([A-Fa-f0-9]{2})?$|', $color ) ) {
			return $color;
		}

		return '';
	}

	/**
	 * Render a select field.
	 *
	 * @param array $args Configuration arguments used by the select field.
	 *
	 * @return void
	 */
	public static function render_select_field( $args ) {
		$defaults = array(
			'name'          => '',
			'options'       => array(),
			'default_value' => '',
			'disabled'      => false,
			'forced_value'  => null,
		);
		$args     = array_merge( $defaults, $args );
		if ( empty( $args['name'] ) || empty( $args['options'] ) ) {
			return;
		}
		$value = ! is_null( $args['forced_value'] ) ? $args['forced_value'] : get_option( $args['name'], $args['default_value'] );

		// Make sure $value is evaluated as boolean for true/false options.
		$is_bool = false;
		if ( count( $args['options'] ) === 2 ) {
			reset( $args['options'] );
			$first = key( $args['options'] );
			end( $args['options'] );
			$last = key( $args['options'] );
			reset( $args['options'] );
			if ( is_numeric( $first ) && is_numeric( $last ) ) {
				$value   = (bool) $value;
				$is_bool = true;
			}
		}
		?>
		<select name="<?php echo esc_attr( $args['name'] ); ?>"
		<?php
		if ( ! empty( $args['disabled'] ) ) {
			echo 'disabled';
		}
		?>
		>
		<?php
		foreach ( $args['options'] as $key => $label ) :
			$compare_key = $key;
			if ( $is_bool ) {
				$compare_key = (bool) $key;
			}
			?>
				<option value="<?php echo esc_attr( $key ); ?>" <?php selected( $compare_key, $value ); ?>><?php echo esc_html( $label ); ?></option>
			<?php endforeach; ?>
		</select>
		<?php if ( ! empty( $args['description'] ) ) : ?>
			<p class="description"><?php echo wp_kses_post( $args['description'] ); ?></p>
			<?php
			endif;
		if ( ! empty( $args['after_description'] ) && is_callable( $args['after_description'] ) ) {
			$args['after_description']( $args );
		}
	}

	/**
	 * Renders multiselect checkbox field.
	 *
	 * @param array $args Field args.
	 *
	 * @return void
	 */
	public static function render_multiselect_checkboxes_field( $args ) {
		$defaults = array(
			'name'           => '',
			'options'        => array(),
			'default_values' => array(),
			'description'    => '',
		);
		$args     = array_merge( $defaults, $args );
		if ( empty( $args['name'] ) || empty( $args['options'] ) ) {
			return;
		}

		$args['checked'] = get_option( $args['name'], $args['default_values'] );

		Tasty_Recipes::echo_template_part(
			'admin/settings/multiselect-checkbox',
			apply_filters( 'tasty_recipes_admin_settings_field_args', $args )
		);
	}

	/**
	 * Renders the card buttons field.
	 *
	 * @return void
	 */
	public static function render_card_buttons_field() {

		$options = array(
			''            => array(
				'label' => __( 'Off', 'tasty-recipes' ),
			),
			'print'       => array(
				'label' => __( 'Print', 'tasty-recipes' ),
			),
			'pin'         => array(
				'label' => __( 'Pin this recipe', 'tasty-recipes' ),
			),
			'mediavine'   => array(
				'label' => __( 'Mediavine Grow.me Save', 'tasty-recipes' ),
			),
			'slickstream' => array(
				'label'    => __( 'Slickstream Favorites', 'tasty-recipes' ),
				'disabled' => true,
			),
		);

		Integrations\Slickstream::init();
		$options = apply_filters( 'tasty_recipes_button_options', $options );

		$value = Tasty_Recipes::get_card_button_settings();
		?>
		<div>
			<?php foreach ( array( 'first', 'second' ) as $position ) : ?>
				<select name="<?php echo esc_attr( Tasty_Recipes::CARD_BUTTONS_OPTION . '[' . $position . ']' ); ?>">
				<?php foreach ( $options as $key => $option ) : ?>
					<option value="<?php echo esc_attr( $key ); ?>"
						<?php selected( $key, $value[ $position ] ); ?>
						<?php echo empty( $option['disabled'] ) ? '' : ' disabled'; ?>
						>
						<?php echo esc_html( $option['label'] ); ?>
					</option>
					<?php endforeach; ?>
				</select>
			<?php endforeach; ?>
		</div>
		<?php
	}

	/**
	 * Gets the license check object.
	 *
	 * @deprecated 3.10
	 *
	 * @return bool|object
	 */
	public static function get_license_check() {
		_deprecated_function( __METHOD__, '3.10' );

		return false;
	}

	/**
	 * Check if Cook Mode can be enabled in this site or not.
	 *
	 * @since 3.8
	 *
	 * @return bool
	 */
	public static function can_enable_cook_mode() {
		/**
		 * Filters the capability of enabling cook mode be enabled in this site.
		 *
		 * @param bool $can_enable True to enable, false otherwise.
		 */
		return (bool) apply_filters( 'tasty_recipes_enable_cook_mode', is_ssl() );
	}

	/**
	 * Is Cook Mode enabled or not.
	 *
	 * @since 3.8
	 *
	 * @return bool
	 */
	public static function is_cook_mode_enabled() {
		return (bool) get_option( Tasty_Recipes::COOK_MODE_OPTION, self::can_enable_cook_mode() );
	}

	/**
	 * Show SSL certificate error message.
	 *
	 * @since 3.8
	 *
	 * @return void
	 */
	public static function show_ssl_error() {
		if ( self::can_enable_cook_mode() ) {
			return;
		}
		$error_message = __( 'Cook Mode requires SSL (HTTPS)', 'tasty-recipes' );
		echo "<div class='tasty-recipes-settings-error'>" . esc_html( $error_message ) . '</div>';
	}

	/**
	 * Get Cook Mode helper message.
	 *
	 * @since 3.8
	 *
	 * @return string
	 */
	public static function get_cook_mode_helper() {
		return get_option( Tasty_Recipes::COOK_MODE_HELPER_OPTION, __( 'Prevent your screen from going dark', 'tasty-recipes' ) );
	}

	/**
	 * Validate the current user capability and exit with json error in case of permission denied.
	 *
	 * @since 3.8
	 *
	 * @return void
	 */
	public static function validate_ajax_capability() {
		if ( current_user_can( self::CAPABILITY ) ) {
			return;
		}

		Utils::send_json_error( __( 'You don\'t have permission to do this.', 'tasty-recipes' ) );
	}

	/**
	 * Check if instacart option is enabled and affiliate ID option is not empty.
	 *
	 * @since 3.8
	 *
	 * @return bool
	 */
	public static function can_show_instacart_button() {
		return (bool) get_option( Tasty_Recipes::INSTACART_OPTION, false );
	}

	/**
	 * Print instacart image in admin instead of real iframe.
	 *
	 * @since 3.8
	 *
	 * @return void
	 */
	public static function print_mock_instacart_html() {
		if ( ! self::can_show_instacart_button() ) {
			return;
		}

		echo '<p><a href=""><img src="' . esc_url( plugins_url( 'assets/images/instacart.svg', TASTY_RECIPES_PLUGIN_FILE ) ) . '"></a></p>';
	}

	/**
	 * Filter quick links admin field to handle the removed item "both"
	 * and cast the string option to an array.
	 *
	 * @param array $args Admin field args.
	 *
	 * @return array
	 */
	public static function filter_quick_links_backward_compatibility( $args ) {
		if ( Tasty_Recipes::QUICK_LINKS_OPTION !== $args['name'] ) {
			return $args;
		}

		$args['checked'] = self::handle_quick_links_backward_compatibility( $args['checked'] );

		return $args;
	}

	/**
	 * Handle the removed item "both" and cast the string option to an array.
	 *
	 * @param mixed $option_value Option value to be handled.
	 *
	 * @return array|string[]
	 */
	public static function handle_quick_links_backward_compatibility( $option_value ) {
		if ( empty( $option_value ) ) {
			return array();
		}

		if ( 'both' === $option_value ) {
			return array(
				'jump',
				'print',
			);
		}

		return (array) $option_value;
	}
}
